"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _babelPluginMacros = require("babel-plugin-macros");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _default = (0, _babelPluginMacros.createMacro)(reanimatedMacro);

exports.default = _default;

function reanimatedMacro({
  references,
  state,
  babel
}) {
  const libraryIdentifier = state.file.path.scope.generateUidIdentifier('reanimated');
  references.default.forEach(referencePath => {
    if (referencePath.parentPath.type === 'TaggedTemplateExpression') {
      /**
       * looks like always inside quasi we have n expressions and n+1 quasis
       * we can create simple template from that by leaving EXPRESSION_X
       * and filling them later one in template.
       * This template is then wrapped in CallExpression so next if can take it
       * and do proper transformations.
       */
      const quasi = referencePath.parent.quasi;
      const tagName = referencePath.container.tag.name;
      const templateCore = quasi.quasis.reduce((acc, curr, i) => {
        if (quasi.quasis.length - 1 !== i) {
          return acc.concat(curr.value.raw).concat(`EXPRESSION_${i}`);
        }

        return acc.concat(curr.value.raw);
      }, '');
      const templateConfig = quasi.expressions.reduce((acc, curr, i) => _objectSpread({}, acc, {
        [`EXPRESSION_${i}`]: curr
      }), {});
      const ast = babel.template(`${tagName}(${templateCore})`)(templateConfig);
      referencePath.parentPath.replaceWith(ast);
    }

    if (referencePath.parentPath.type === 'CallExpression') {
      const _referencePath$parent = referencePath.parentPath.get('arguments'),
            _referencePath$parent2 = _slicedToArray(_referencePath$parent, 1),
            argumentPath = _referencePath$parent2[0];

      genericReplace(argumentPath, state, babel, libraryIdentifier);
    } else {
      throw new Error(`reanimated-math.macro can only be used as tagged template expression or function call. You tried ${referencePath.parentPath.type}.`);
    }
  }); // add reanimated import

  state.file.path.node.body.unshift(babel.types.importDeclaration([babel.types.importDefaultSpecifier(libraryIdentifier)], babel.types.stringLiteral('react-native-reanimated')));
}

function genericReplace(argumentPath, state, babel, libraryIdentifier) {
  const visitor = {
    BinaryExpression(path) {
      const ast = babel.template('LIBRARY.METHOD(LEFT_EXPRESSION, RIGHT_EXPRESSION)')({
        LIBRARY: libraryIdentifier,
        METHOD: babel.types.identifier(binaryOperators[path.node.operator]),
        LEFT_EXPRESSION: path.node.left,
        RIGHT_EXPRESSION: path.node.right
      });
      path.replaceWith(ast);
    },

    CallExpression(path) {
      if (path.node.callee.type === 'Identifier') {
        const ast = babel.template('LIBRARY.METHOD(ARGUMENTS)')({
          LIBRARY: libraryIdentifier,
          METHOD: babel.types.identifier(path.node.callee.name),
          ARGUMENTS: path.node.arguments
        });
        path.replaceWith(ast);
      }
    },

    UnaryExpression(path) {
      if (path.node.operator === '-') {
        // for unary minus operator: -var
        const ast = babel.template('LIBRARY.METHOD(ARGUMENT, ARGUMENT2)')({
          LIBRARY: libraryIdentifier,
          METHOD: babel.types.identifier('multiply'),
          ARGUMENT: path.node.argument,
          ARGUMENT2: '-1'
        });
        path.replaceWith(ast);
      }
    }

  };
  argumentPath.parentPath.traverse(visitor);
  argumentPath.parentPath.replaceWith(argumentPath);
}

const binaryOperators = {
  '+': 'add',
  '-': 'sub',
  '*': 'multiply',
  '/': 'divide',
  '**': 'pow',
  '%': 'mod'
};